/* Copyright (C) 2022 INRIA

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <pari/pari.h>
#include <complex.h>
#include <sys/types.h>
#include <dirent.h>

#define NVAR   12   /* Nombre de variables.                */
#define NSCH  868   /* Nombre de schémas valides.          */
#define NEQ   178   /* Nombre d'équations dans le système. */
#define NMON   34   /* Nombre de monômes à calculer.       */
#define EMAX    6   /* Exposant maximal d'une variable.    */
#define WIDTH  40   /* Précision des résultats flottants.  */

/* Schémas pouvant correspondre à des corps A4 ou S4 ; les 1 indiquent
 * la position des variables non nulles. */
static const char schtable[NSCH][NVAR] = {
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1},
    {0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1},
    {1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0},
    {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0},
    {1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1},
    {1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1},
    {0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1},
    {0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 0},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0},
    {0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1},
    {1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1},
    {1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1},
    {0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1},
    {1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1},
    {1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1},
    {1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0},
    {1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1},
    {1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1},
    {1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
    {0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1},
    {1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1},
    {1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1},
    {1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1},
    {0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1},
    {1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0},
    {1, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1},
    {1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1},
    {0, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0},
    {1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {1, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1},
    {0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0},
    {0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1},
    {0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0},
    {1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1},
    {0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1},
    {1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0},
    {1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1},
    {1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1},
    {1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1},
    {0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0},
    {0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {1, 1, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
    {1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1},
    {1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0},
    {1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1},
    {1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1},
    {1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1},
    {0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1},
    {1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1},
    {0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1},
    {1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1},
    {0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0},
    {0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1},
    {1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1},
    {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1},
    {0, 1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1},
    {1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
    {1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0},
    {1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 1, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1},
    {1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 1},
    {1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0},
    {1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0},
    {1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0},
    {1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0},
    {0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1},
    {1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1},
    {1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
    {1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1},
    {1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1},
    {1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0},
    {0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1},
    {1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0},
    {1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1},
    {1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0, 1},
};

/* Changements de signes autorisés pour chaque schéma ; les 1
 * indiquent la position des variables pouvant être négatives. Le
 * nombre de signes libres est égal au nombre de variables non nulles
 * moins 4. */
static const char sgntable[NSCH][NVAR] = {
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
};

/* Exposants des monômes du système d'équations. */
static const char eqtable[NEQ][NVAR] = {
    {0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 2, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 2, 1, 1, 2, 0, 0, 0, 0},
    {0, 0, 0, 0, 2, 1, 2, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 2, 1, 3, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 3, 0, 1, 1, 1, 0, 0, 0},
    {0, 0, 0, 0, 3, 0, 1, 2, 0, 0, 0, 0},
    {0, 0, 0, 0, 3, 0, 2, 0, 0, 0, 1, 0},
    {0, 0, 0, 0, 3, 0, 2, 0, 0, 1, 0, 0},
    {0, 0, 0, 0, 3, 0, 2, 0, 1, 0, 0, 0},
    {0, 0, 0, 0, 6, 0, 3, 0, 2, 1, 0, 0},
    {0, 0, 0, 0, 6, 0, 3, 2, 0, 0, 0, 1},
    {0, 0, 0, 0, 6, 0, 4, 0, 0, 1, 0, 1},
    {0, 0, 0, 0, 6, 0, 5, 0, 0, 0, 0, 1},
    {0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 2, 1, 2, 0, 0, 0, 0},
    {0, 0, 0, 1, 0, 2, 2, 0, 0, 1, 0, 0},
    {0, 0, 0, 1, 0, 2, 3, 0, 0, 0, 0, 0},
    {0, 0, 0, 1, 2, 0, 1, 0, 2, 0, 0, 0},
    {0, 0, 0, 1, 2, 0, 2, 0, 0, 0, 0, 1},
    {0, 0, 0, 2, 0, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 0, 2, 0, 1, 1, 0, 2, 0, 0, 0},
    {0, 0, 0, 2, 0, 1, 2, 0, 0, 0, 0, 1},
    {0, 0, 0, 3, 0, 0, 1, 0, 2, 0, 0, 0},
    {0, 0, 0, 3, 0, 0, 1, 2, 0, 0, 0, 0},
    {0, 0, 0, 3, 0, 0, 2, 0, 0, 0, 0, 1},
    {0, 0, 0, 3, 0, 0, 2, 0, 0, 0, 1, 0},
    {0, 0, 0, 3, 0, 0, 2, 0, 0, 1, 0, 0},
    {0, 0, 0, 3, 0, 3, 4, 0, 0, 0, 2, 0},
    {0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0},
    {0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0},
    {0, 0, 1, 0, 1, 1, 0, 3, 0, 0, 0, 0},
    {0, 0, 1, 0, 2, 0, 0, 2, 1, 0, 0, 0},
    {0, 0, 1, 0, 2, 0, 0, 3, 0, 0, 0, 0},
    {0, 0, 1, 2, 0, 0, 0, 0, 3, 0, 0, 0},
    {0, 0, 1, 2, 0, 0, 0, 3, 0, 0, 0, 0},
    {0, 0, 2, 0, 0, 0, 0, 0, 0, 1, 0, 0},
    {0, 0, 2, 0, 0, 0, 0, 1, 0, 0, 0, 0},
    {0, 0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0},
    {0, 0, 2, 0, 0, 1, 0, 2, 0, 1, 0, 0},
    {0, 0, 2, 0, 0, 1, 0, 3, 0, 0, 0, 0},
    {0, 0, 2, 0, 0, 1, 1, 0, 0, 2, 0, 0},
    {0, 0, 2, 0, 0, 1, 3, 0, 0, 0, 0, 0},
    {0, 0, 2, 0, 1, 0, 0, 2, 0, 0, 1, 0},
    {0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 2, 1, 0, 0, 0, 2, 0, 0, 0, 1},
    {0, 0, 2, 1, 0, 0, 1, 0, 0, 0, 2, 0},
    {0, 0, 2, 1, 0, 3, 0, 6, 0, 0, 0, 0},
    {0, 0, 2, 4, 0, 0, 3, 0, 0, 0, 0, 3},
    {0, 0, 3, 0, 0, 0, 0, 0, 1, 2, 0, 0},
    {0, 0, 3, 0, 0, 0, 0, 1, 0, 1, 1, 0},
    {0, 0, 3, 0, 0, 0, 0, 1, 0, 2, 0, 0},
    {0, 0, 3, 0, 0, 0, 0, 2, 0, 0, 1, 0},
    {0, 0, 3, 0, 0, 0, 0, 2, 1, 0, 0, 0},
    {0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 1, 0},
    {0, 0, 3, 0, 0, 0, 2, 0, 1, 0, 0, 0},
    {0, 0, 3, 0, 3, 0, 0, 5, 0, 0, 0, 1},
    {0, 0, 5, 1, 0, 0, 0, 3, 0, 0, 3, 0},
    {0, 0, 6, 0, 0, 0, 0, 2, 0, 3, 0, 1},
    {0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 1},
    {0, 0, 6, 0, 0, 0, 1, 0, 0, 3, 2, 0},
    {0, 0, 6, 0, 0, 0, 1, 0, 0, 4, 0, 1},
    {0, 0, 6, 0, 0, 0, 1, 0, 0, 5, 0, 0},
    {0, 0, 6, 0, 0, 0, 5, 0, 0, 0, 0, 1},
    {0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0},
    {0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
    {0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0},
    {0, 1, 0, 0, 0, 2, 0, 3, 0, 0, 0, 0},
    {0, 1, 0, 0, 0, 2, 3, 0, 0, 0, 0, 0},
    {0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0},
    {0, 1, 0, 0, 2, 0, 0, 1, 2, 0, 0, 0},
    {0, 1, 0, 0, 2, 0, 0, 3, 0, 0, 0, 0},
    {0, 1, 0, 1, 1, 0, 0, 0, 3, 0, 0, 0},
    {0, 1, 0, 2, 0, 0, 0, 0, 3, 0, 0, 0},
    {0, 1, 0, 2, 0, 0, 0, 3, 0, 0, 0, 0},
    {0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0},
    {0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0},
    {0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1, 2, 0, 0, 0, 0, 1, 0, 0, 2, 0},
    {0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0},
    {0, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0},
    {0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {0, 2, 0, 0, 0, 1, 0, 0, 2, 1, 0, 0},
    {0, 2, 0, 0, 0, 1, 1, 0, 0, 0, 2, 0},
    {0, 2, 0, 0, 0, 4, 3, 0, 0, 3, 0, 0},
    {0, 2, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0},
    {0, 2, 0, 0, 1, 0, 0, 0, 2, 0, 1, 0},
    {0, 2, 0, 0, 1, 0, 0, 0, 3, 0, 0, 0},
    {0, 2, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2, 0, 1, 0, 0, 0, 0, 2, 0, 0, 1},
    {0, 2, 0, 1, 0, 0, 1, 0, 0, 0, 0, 2},
    {0, 2, 0, 3, 0, 1, 0, 0, 6, 0, 0, 0},
    {0, 2, 1, 0, 0, 0, 0, 0, 1, 0, 2, 0},
    {0, 2, 1, 0, 0, 0, 0, 0, 3, 0, 0, 0},
    {0, 3, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1},
    {0, 3, 0, 0, 0, 0, 0, 0, 1, 0, 2, 0},
    {0, 3, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0},
    {0, 3, 0, 0, 0, 0, 0, 0, 2, 0, 0, 1},
    {0, 3, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2},
    {0, 3, 0, 0, 0, 0, 0, 1, 0, 0, 2, 0},
    {0, 3, 0, 0, 0, 0, 0, 1, 0, 2, 0, 0},
    {0, 3, 0, 0, 0, 0, 1, 0, 0, 0, 0, 2},
    {0, 3, 0, 0, 3, 0, 0, 0, 5, 1, 0, 0},
    {0, 3, 3, 0, 0, 0, 1, 0, 0, 0, 5, 0},
    {0, 5, 0, 0, 0, 1, 0, 0, 3, 0, 3, 0},
    {0, 5, 0, 0, 0, 1, 0, 0, 6, 0, 0, 0},
    {0, 6, 0, 0, 0, 0, 0, 0, 2, 1, 0, 3},
    {0, 6, 0, 0, 0, 0, 1, 0, 0, 0, 2, 3},
    {0, 6, 0, 0, 0, 0, 1, 0, 0, 0, 5, 0},
    {0, 6, 0, 0, 0, 0, 1, 0, 0, 1, 0, 4},
    {0, 6, 0, 0, 0, 0, 1, 0, 0, 5, 0, 0},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0},
    {1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0},
    {1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0},
    {1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0},
    {1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},
    {1, 0, 0, 0, 0, 2, 0, 2, 0, 1, 0, 0},
    {1, 0, 0, 0, 0, 2, 0, 3, 0, 0, 0, 0},
    {1, 0, 0, 0, 0, 2, 1, 0, 0, 2, 0, 0},
    {1, 0, 0, 0, 0, 2, 3, 0, 0, 0, 0, 0},
    {1, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0},
    {1, 0, 0, 0, 2, 0, 0, 0, 2, 1, 0, 0},
    {1, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 1},
    {1, 0, 0, 0, 2, 3, 0, 6, 0, 0, 0, 0},
    {1, 0, 0, 0, 5, 0, 0, 3, 3, 0, 0, 0},
    {1, 0, 0, 0, 5, 0, 0, 6, 0, 0, 0, 0},
    {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},
    {1, 0, 0, 1, 0, 4, 0, 6, 0, 0, 0, 0},
    {1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0},
    {1, 0, 0, 2, 0, 0, 0, 0, 2, 0, 0, 1},
    {1, 0, 0, 2, 0, 0, 1, 0, 0, 0, 0, 2},
    {1, 0, 0, 3, 2, 0, 0, 0, 6, 0, 0, 0},
    {1, 0, 0, 4, 0, 1, 0, 0, 6, 0, 0, 0},
    {1, 0, 0, 5, 0, 0, 0, 0, 6, 0, 0, 0},
    {1, 0, 0, 5, 0, 0, 0, 6, 0, 0, 0, 0},
    {1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1, 0, 2, 0, 0, 0, 0, 0, 0, 1, 2, 0},
    {1, 0, 2, 0, 0, 0, 0, 0, 0, 2, 0, 1},
    {1, 0, 2, 0, 0, 0, 0, 0, 0, 3, 0, 0},
    {1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 3, 0},
    {1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1},
    {1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0},
    {1, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 2},
    {1, 2, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0},
    {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {2, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},
    {2, 0, 0, 0, 0, 1, 0, 0, 0, 1, 2, 0},
    {2, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 1},
    {2, 0, 0, 0, 0, 1, 0, 0, 0, 3, 0, 0},
    {2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 3, 0},
    {2, 0, 0, 0, 1, 0, 0, 0, 0, 3, 0, 0},
    {2, 0, 0, 0, 1, 0, 0, 0, 3, 0, 0, 0},
    {2, 0, 0, 1, 0, 0, 0, 0, 0, 0, 2, 1},
    {2, 0, 0, 1, 0, 0, 0, 0, 0, 0, 3, 0},
    {2, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 2},
    {2, 0, 0, 1, 0, 0, 0, 0, 0, 3, 0, 0},
    {2, 0, 1, 0, 0, 0, 0, 0, 0, 0, 3, 0},
    {2, 0, 1, 0, 0, 0, 0, 0, 3, 0, 0, 0},
    {3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1},
    {3, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 2},
    {3, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 1},
    {3, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2},
    {3, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 2},
    {3, 0, 0, 0, 0, 3, 0, 0, 2, 4, 0, 0},
    {3, 0, 0, 3, 0, 0, 0, 2, 0, 0, 0, 4},
    {3, 0, 2, 1, 0, 0, 0, 0, 0, 0, 6, 0},
    {3, 2, 0, 0, 0, 1, 0, 0, 0, 0, 6, 0},
    {4, 0, 0, 0, 2, 0, 0, 0, 0, 3, 0, 3},
    {4, 0, 0, 1, 0, 1, 0, 0, 0, 0, 6, 0},
    {5, 0, 0, 0, 0, 1, 0, 0, 0, 0, 6, 0},
    {5, 0, 0, 0, 0, 1, 0, 0, 6, 0, 0, 0},
};

/* Exposants des monômes à calculer. */
static const char montable[NMON][NVAR] = {
    {1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0},   /* a11 * a22 * a33 */
    {0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0},   /* a12 * a13 * a23 */
    {1, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0},   /* a11 * a23^2     */
    {0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0},   /* a22 * a13^2     */
    {0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0},   /* a33 * a12^2     */
    {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},   /* a11 * a22 * b33 */
    {1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0},   /* a11 * b22 * a33 */
    {0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0},   /* b11 * a22 * a33 */
    {0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0},   /* a12 * a13 * b23 */
    {0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0},   /* a12 * b13 * a23 */
    {0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0},   /* b12 * a13 * a23 */
    {0, 0, 0, 0, 2, 0, 1, 0, 0, 0, 0, 0},   /* b11 * a23^2     */
    {0, 0, 2, 0, 0, 0, 0, 0, 0, 1, 0, 0},   /* b22 * a13^2     */
    {0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},   /* b33 * a12^2     */
    {1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0},   /* a11 * a23 * b23 */
    {0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0},   /* a22 * a13 * b13 */
    {0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0},   /* a33 * a12 * b12 */
    {0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0},   /* b11 * b22 * a33 */
    {0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1},   /* b11 * a22 * b33 */
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},   /* a11 * b22 * b33 */
    {0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0},   /* b12 * b13 * a23 */
    {0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0},   /* b12 * a13 * b23 */
    {0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0},   /* a12 * b13 * b23 */
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0},   /* a11 * b23^2     */
    {0, 0, 0, 1, 0, 0, 0, 0, 2, 0, 0, 0},   /* a22 * b13^2     */
    {0, 0, 0, 0, 0, 1, 0, 2, 0, 0, 0, 0},   /* a33 * b12^2     */
    {0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0},   /* b11 * b23 * a23 */
    {0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0},   /* b22 * b13 * a13 */
    {0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},   /* b33 * b12 * a12 */
    {0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1},   /* b11 * b22 * b33 */
    {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0},   /* b12 * b13 * b23 */
    {0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0},   /* b11 * b23^2     */
    {0, 0, 0, 0, 0, 0, 0, 0, 2, 1, 0, 0},   /* b22 * b13^2     */
    {0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1},   /* b33 * b12^2     */
};

/* Coefficients des monômes à calculer. */
static long coeftable[NMON] = {
    4, 1, -1, -1, -1,
    4, 4, 4, 1, 1, 1, -1, -1, -1, -2, -2, -2,
    4, 4, 4, 1, 1, 1, -1, -1, -1, -2, -2, -2,
    4, 1, -1, -1, -1,
};

/* Position des changements successifs obtenus en écrivant tous les
 * entiers de NVAR - 4 bits avec le code de Gray. */
static int gray[1 << (NVAR - 4)] = {
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 5,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 6,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 5,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 7,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 5,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 6,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 5,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 4,
    0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, 8,
};

/* Valeurs des variables. */
static long *var[NVAR];   /* Valeurs absolues.                   */
static int *vsgn[NVAR];   /* Signes : 0 = positif, -1 = négatif. */

/* Valeurs des monômes. */
static long mon[NMON];   /* Valeurs absolues.                   */
static int msgn[NMON];   /* Signes : 0 = positif, -1 = négatif. */

/* Tableaux utilisés lors du parcours de l'arbre des solutions
 * correspondant à un schéma donné. Seules interviennent les variables
 * non nulles. */
static long nz[NVAR];       /* Valeurs absolues des variables non nulles.   */
static int np[NVAR - 4];    /* Signes des variables non positives.          */
static int eqcnt[NVAR];     /* Nombre d'équations à utiliser.               */
static int rhscnt[NVAR];    /* Nombre de membres droits à mettre à jour.    */
static int coefcnt[NVAR];   /* Nombre de coefficients à mettre à jour.      */
static int eqexp[NEQ][NVAR];      /* Exposants des équations utilisées.     */
static int rhsexp[NEQ][NVAR];     /* Exposants des équations mises à jour.  */
static int coefexp[NMON][NVAR];   /* Exposants des monômes mis à jour.      */
static long *eqrhs[NEQ][NVAR];    /* Membres droits des équations utilisés. */
static long *oldrhs[NEQ][NVAR];     /* Anciens membres droits.              */
static long *newrhs[NEQ][NVAR];     /* Nouveaux membres droits.             */
static long *oldcoef[NMON][NVAR];   /* Anciens coefficients.                */
static long *newcoef[NMON][NVAR];   /* Nouveaux coefficients.               */
static int eqemax[NVAR];      /* Exposant maximal des équations utilisées.  */
static int upemax[NVAR];      /* Exposant maximal pour les mises à jour.    */
static int mscnt[NVAR];       /* Nombre de signes de monômes à changer.     */
static int *ms[NMON][NVAR];   /* Signes de monômes à changer.               */

/* Fichier temporaire pour enregistrer les résultats. */
static FILE *tmpfp;

#ifdef cnt
static long initcnt;   /* Points issus du parcours de l'arbre.           */
static long disccnt;   /* Points vérifiant la borne sur le discriminant. */
static long red2cnt;   /* Points de covariant 2x2 réduit.                */
static long red3cnt;   /* Points de covariant 3x3 réduit.                */
static long irrcnt;    /* Points irréductibles.                          */
static long maxcnt;    /* Points maximaux.                               */
#endif

/* Représentation d'un corps quartique. */
struct elt {
    long disc3;     /* Opposé du triple du discriminant.  */
    long v[NVAR];   /* Valeurs des variables.             */
    long p[4];      /* Coefficients du covariant cubique. */
    long r[5];      /* Coefficients du résultant.         */
};

/* Adresse d'un élément enregistré dans un fichier temporaire. */
struct addr {
    long disc3;    /* Opposé du triple du discriminant. */
    FILE *tmpfp;   /* Fichier temporaire.               */
    long offset;   /* Position dans le fichier.         */
};

/* Lecture de la ligne de commande, lancement du programme. */
static void usage (void);
int main (int argc, char **argv);

/* Parcours de l'arbre des solutions correspondant au schéma sch. */
static void init (long x3, int mod, int rem, int sch);
static void init_scheme (long x3, int mod, int rem, int sch,
                         int elast[NEQ], int mlast[NMON],
                         long *rhs[NEQ][NVAR], long *coef[NMON][NVAR]);
static void browse (long x3, int mod, int rem, int nzcnt, int npcnt, int j);
static inline void choose_sgn (long x3, int npcnt);
static void process (long x3);

/* Filtrage des solutions. */
static inline long compute_disc3 (long p[4], long s[4][4], long q[3]);
static inline int red2 (long p[4], long s[4][4], long q[3], long disc3);
static inline void compute_v (long v[NVAR]);
static inline void compute_zc (long v[NVAR], long p[4], long s[4][4],
                               long disc3, double complex zc[3][4]);
static inline void compute_cov3 (double complex zc[3][4], double cov3[6]);
static inline int red3 (double cov3[6]);
static inline int irred (long v[NVAR], long p[4], long r[5]);
static inline int maximal (long disc3, long r[5]);

/* Calcul du zéro-cycle associé à un point donné. */
static inline double root3 (long p[4], long s[4][4], long disc3);
static inline double diagonalize (double degen[6], double evec[3][3]);
static inline double apply1 (long v[6], double evec[3][3], int j);
static inline double apply2 (long v[6], double evec[3][3],  int j1, int j2);
static inline void roots2 (double a, double b, double c, double complex z[2]);
static inline void croots2 (double complex a, double complex b,
                            double complex c, double complex z[2]);
static inline void fill_zc (double evec[3][3], double complex x,
                            double complex y, double complex zc[3][4], int j);

/* Diagonalisation d'une forme quadratique dégénérée. */
static inline void set_hcom4 (double degen[6], double hcom4[6]);
static inline void set_dcom4 (double degen[6],
                              double hcom4[6], double dcom4[3]);
static void find_evec (double degen[6], double hcom4[6], double dcom4[3],
                       double evec[3][3], int j);

/* Élimination des doublons et écriture des résultats. */
static long count_elements_tmpfp (FILE *tmpfp, struct addr *addresses);
static long count_elements (struct addr *addresses);
static void read_element (struct addr *a, struct elt *e);
static int compare_addresses (const void *p1, const void *p2);
static void mark_duplicates (struct elt *e, int *d, int ecnt);
static void write_elt (FILE *outfile, struct elt *e);
static int write_disc3 (struct addr *addresses, long ecnt, long disc3,
                        FILE *outfile);
static void write_res (void);

/* Fonctions auxiliaires. */
static inline long sgnsum (long *abs, int *sgn, int length);
static inline double csprod (double complex z1, double complex z2);
static inline int relaxed_gt (double x1, double x2);
static GEN make_pol (long *coefs, int n);

static void usage (void)
{
    fputs ("Recherche des solutions : enum -s borne [module reste] schéma\n"
           "Écriture des résultats :  enum -w\n", stderr);
}

int main (int argc, char **argv)
{
    int error  = 0;   /* Ligne de commande incorrecte.           */
    int search = 0;   /* Recherche des solutions.                */
    long x3;          /* Triple de la borne sur le discriminant. */
    int mod = 1;      /* Module.                                 */
    int rem = 0;      /* Reste.                                  */
    int sch;          /* Schéma à parcourir.                     */

    switch (argc) {
    case 6:
        mod = atoi (argv[3]);
        rem = atoi (argv[4]);
        if (mod <= 0 || rem < 0 || rem >= mod)
            error = 1;
    case 4:
        sch = atoi (argv[argc - 1]);
        if (sch < 0 || sch >= NSCH ||
            strcmp (argv[1], "-s") || (x3 = 3 * atol (argv[2])) <= 0)
            error = 1;
        search = 1;
        break;
    case 2:
        if (strcmp (argv[1], "-w"))
            error = 1;
        break;
    default:
        error = 1;
        break;
    }
    if (error) {
        usage ();
        return 1;
    }
    pari_init (1000000, 100000);

    if (search)
        init (x3, mod, rem, sch);
    else
        write_res ();

#ifdef cnt
    if (search) {
        int j;

        printf ("%3d / %3d de %3d - ", rem, mod, sch);
        for (j = 0; j < NVAR; j++)
            printf ("%d ", schtable[sch][j]);
        printf (": init = %ld  disc = %ld  red2 = %ld  red3 = %ld  "
                "irr = %ld  max = %ld\n",
                initcnt, disccnt, red2cnt, red3cnt, irrcnt, maxcnt);
    }
#endif

    return 0;
}

/* Initialisation des membres droits des équations, et construction de
 * structures de données permettant de ne mettre à jour les membres
 * droits des équations et les coefficients des monômes qu'en cas de
 * nécessité. */
static void init (long x3, int mod, int rem, int sch)
{
    int i, j;
    double all, diag;
    const double a = pow (2, 10. / 3.) * pow (3, -19. / 12.);
    int elast[NEQ];                   /* Dernières variables des équations. */
    int mlast[NMON];                  /* Dernières variables des monômes.   */
    long *rhs[NEQ][NVAR];             /* Emplacement des membres droits.    */
    long *coef[NMON][NVAR];           /* Emplacement des coefficients.      */
    long rhsstore[NEQ][NVAR];         /* Stockage des membres droits.       */
    long coefstore[NMON][NVAR - 1];   /* Stockage des coefficients.         */
    char tmpname[21] = "tmp_";        /* Nom de fichier temporaire.         */

    for (i = 0; i < NEQ; i++) {
        for (elast[i] = NVAR - 1; !eqtable[i][elast[i]]; elast[i]--);

        all = 0;
        for (j = 0; j < NVAR; j++)
            all += eqtable[i][j];
        diag = eqtable[i][0] + eqtable[i][3] + eqtable[i][5]
            + eqtable[i][6] + eqtable[i][9] + eqtable[i][11];
        rhsstore[i][0] = pow (2, -diag) * pow (a, all) * pow (x3, all / NVAR);

        rhs[i][0] = &rhsstore[i][0];
        for (j = 0; j < NVAR - 1; j++)
            rhs[i][j + 1] = eqtable[i][j] ? &rhsstore[i][j + 1] : rhs[i][j];
    }
    for (i = 0; i < NMON; i++) {
        for (mlast[i] = NVAR - 1; !montable[i][mlast[i]]; mlast[i]--);

        coef[i][0] = &coeftable[i];
        for (j = 0; j < NVAR - 1; j++)
            coef[i][j + 1] = montable[i][j] ? &coefstore[i][j] : coef[i][j];
    }
    sprintf (tmpname + 4, "%.8x%.8x", rem, sch);
    if (!(tmpfp = fopen (tmpname, "wb")))
        exit (1);

    init_scheme (x3, mod, rem, sch, elast, mlast, rhs, coef);

    fclose (tmpfp);
}

/* Préparation du parcours de l'arbre correspondant au schéma sch. */
static void init_scheme (long x3, int mod, int rem, int sch,
                         int elast[NEQ], int mlast[NMON],
                         long *rhs[NEQ][NVAR], long *coef[NMON][NVAR])
{
    int i, j;
    int n;             /* Nombre de variables non nulles.        */
    int s;             /* Nombre de signes libres.               */
    int ezero[NEQ];    /* Équations nulles.                      */
    int mzero[NMON];   /* Monômes nuls.                          */
    long zstore = 0;   /* Valeurs absolues des variables nulles. */
    int pstore = 0;    /* Signes des variables positives.        */

    for (i = 0; i < NEQ; i++) {
        ezero[i] = 0;
        for (j = 0; j < NVAR; j++)
            if (!schtable[sch][j] && eqtable[i][j]) {
                ezero[i] = 1;
                break;
            }
    }
    for (i = 0; i < NMON; i++) {
        mzero[i] = 0;
        for (j = 0; j < NVAR; j++)
            if (!schtable[sch][j] && montable[i][j]) {
                mzero[i] = 1;
                mon[i] = 0;
                break;
            }
    }
    n = s = 0;

    for (j = 0; j < NVAR; j++) {
        if (!schtable[sch][j]) {
            var[j] = &zstore;
            vsgn[j] = &pstore;
            continue;
        }
        var[j] = &nz[n];

        eqcnt[n] = rhscnt[n] = coefcnt[n] = 0;
        eqemax[n] = upemax[n] = 0;

        for (i = 0; i < NEQ; i++) {
            if (ezero[i] || !eqtable[i][j])
                continue;
            eqexp[eqcnt[n]][n] = eqtable[i][j];
            eqrhs[eqcnt[n]][n] = rhs[i][j];
            eqcnt[n]++;
            if (eqtable[i][j] > eqemax[n])
                eqemax[n] = eqtable[i][j];

            if (j == elast[i])
                continue;
            rhsexp[rhscnt[n]][n] = eqtable[i][j];
            oldrhs[rhscnt[n]][n] = rhs[i][j];
            newrhs[rhscnt[n]][n] = rhs[i][j + 1];
            rhscnt[n]++;
            if (eqtable[i][j] > upemax[n])
                upemax[n] = eqtable[i][j];
        }
        for (i = 0; i < NMON; i++) {
            if (mzero[i] || !montable[i][j])
                continue;
            coefexp[coefcnt[n]][n] = montable[i][j];
            oldcoef[coefcnt[n]][n] = coef[i][j];
            newcoef[coefcnt[n]][n] =
                (j < mlast[i]) ? coef[i][j + 1] : &mon[i];
            coefcnt[n]++;
            if (montable[i][j] > upemax[n])
                upemax[n] = montable[i][j];
        }
        n++;

        if (!sgntable[sch][j]) {
            vsgn[j] = &pstore;
            continue;
        }
        vsgn[j] = &np[s];

        mscnt[s] = 0;
        for (i = 0; i < NMON; i++) {
            if (mzero[i] || !(montable[i][j] % 2))
                continue;
            ms[mscnt[s]][s] = &msgn[i];
            mscnt[s]++;
        }
        s++;
    }
    gray[(1 << s) - 1] = s - 1;
    browse (x3, mod, rem, n, 1 << s, 0);
    gray[(1 << s) - 1] = s;
}

/* Parcours de l'arbre des solutions à partir de la j-ième variable
 * non nulle. On reçoit le nombre de variables non nulles et le nombre
 * de combinaisons de signes. */
static void browse (long x3, int mod, int rem, int nzcnt, int npcnt, int j)
{
    int i;
    long tmp;
    long max[EMAX + 1];
    long pw[EMAX + 1];

    for (i = 1; i <= eqemax[j]; i++)
        max[i] = LONG_MAX;

    for (i = 0; i < eqcnt[j]; i++)
        if (*eqrhs[i][j] < max[eqexp[i][j]])
            max[eqexp[i][j]] = *eqrhs[i][j];

    for (i = 2; i <= eqemax[j]; i++)
        if ((tmp = usqrtn (max[i], i)) < max[1])
            max[1] = tmp;

    for (nz[j] = 1; nz[j] <= max[1]; nz[j]++) {
        if (j == nzcnt - 1 && (rem - nz[j]) % mod)
            continue;

        pw[0] = 1;
        for (i = 1; i <= upemax[j]; i++)
            pw[i] = nz[j] * pw[i - 1];

        for (i = 0; i < rhscnt[j]; i++)
            *newrhs[i][j] = *oldrhs[i][j] / pw[rhsexp[i][j]];

        for (i = 0; i < coefcnt[j]; i++)
            *newcoef[i][j] = *oldcoef[i][j] * pw[coefexp[i][j]];

        if (j < nzcnt - 1)
            browse (x3, mod, rem - nz[j], nzcnt, npcnt, j + 1);
        else
            choose_sgn (x3, npcnt);
    }
}

/* Parcours de tous les signes possibles pour les variables pouvant
 * être négatives. On réinitialise les signes à la fin de
 * l'opération. */
static inline void choose_sgn (long x3, int npcnt)
{
    int i, j, k;

    for (k = 0; k < npcnt; k++) {
        process (x3);
        j = gray[k];
        np[j] = ~np[j];
        for (i = 0; i < mscnt[j]; i++)
            *ms[i][j] = ~*ms[i][j];
    }
}

/* Teste la borne sur le discriminant, la réduction, l'irréductibilité
 * et la maximalité, et sauvegarde les résultats. */
static void process (long x3)
{
    struct elt e;              /* Élément en cours de traitement.         */
    long s[4][4];              /* Produits e.p[i] * e.p[j].               */
    long q[3];                 /* Coefficients du covariant quadratique.  */
    double complex zc[3][4];   /* Zéro-cycle défini par v.                */
    double cov3[6];            /* Coefficients du covariant 3x3.          */

#ifdef cnt
    initcnt++;
#endif

    e.disc3 = compute_disc3 (e.p, s, q);

    if (!e.disc3 || labs (e.disc3) > x3)
        return;

#ifdef cnt
    disccnt++;
#endif

    if (!red2 (e.p, s, q, e.disc3))
        return;

#ifdef cnt
    red2cnt++;
#endif

    compute_v (e.v);
    compute_zc (e.v, e.p, s, e.disc3, zc);
    compute_cov3 (zc, cov3);

    if (!red3 (cov3))
        return;

#ifdef cnt
    red3cnt++;
#endif

    if (!irred (e.v, e.p, e.r))
        return;

#ifdef cnt
    irrcnt++;
#endif

    if (!maximal (e.disc3, e.r))
        return;

#ifdef cnt
    maxcnt++;
#endif

    if (fwrite (&e, sizeof e, 1, tmpfp) < 1)
        exit (1);
}

/* Calcul du covariant cubique, du covariant quadratique et du
 * discriminant. Pour gagner du temps, on renvoie 0 dans le cas où le
 * covariant cubique est réductible. */
static inline long compute_disc3 (long p[4], long s[4][4], long q[3])
{
    if (!(p[0] = sgnsum (mon, msgn, 5)))
        return 0;

    if (!(p[3] = sgnsum (mon + 29, msgn + 29, 5)))
        return 0;

    p[1] = sgnsum (mon + 5, msgn + 5, 12);
    p[2] = sgnsum (mon + 17, msgn + 17, 12);

    if (p[0] + p[2] == p[1] + p[3] || p[0] + p[2] == -(p[1] + p[3]))
        return 0;

    s[0][2] = p[0] * p[2];
    s[0][3] = p[0] * p[3];
    s[1][1] = p[1] * p[1];
    s[1][2] = p[1] * p[2];
    s[1][3] = p[1] * p[3];
    s[2][2] = p[2] * p[2];

    q[0] = s[1][1] - 3 * s[0][2];
    q[1] = s[1][2] - 9 * s[0][3];
    q[2] = s[2][2] - 3 * s[1][3];

    return q[1] * q[1] - 4 * q[0] * q[2];
}

/* Calcule s[0][0] et vérifie les conditions de réduction 2x2. */
static inline int red2 (long p[4], long s[4][4], long q[3], long disc3)
{
    if (disc3 < 0) {
        if (q[0] > q[2] || labs (q[1]) > q[0])
            return 0;

        s[0][0] = p[0] * p[0];
    }
    else {
        double x, y;

        s[0][0] = p[0] * p[0];
        s[3][3] = p[3] * p[3];

        x = q[0] - q[2];
        y = 3 * (s[0][0] - s[3][3]) - s[1][1] + s[2][2];

        if (2 * x * x * x + 3 * y * disc3 > 0)
            return 0;

        s[0][1] = p[0] * p[1];

        x = q[0] + q[1];
        y = 3 * s[0][0] + 6 * (s[0][1] + s[0][2] + s[0][3])
            + 2 * (s[1][1] + s[1][2]);

        if (2 * x * x * x + 3 * y * disc3 < 0)
            return 0;

        x = q[0] - q[1];
        y = 3 * s[0][0] - 6 * (s[0][1] - s[0][2] + s[0][3])
            + 2 * (s[1][1] - s[1][2]);

        if (2 * x * x * x + 3 * y * disc3 < 0)
            return 0;
    }
    return 1;
}

/* Calcule les valeurs des variables. */
static inline void compute_v (long v[NVAR])
{
    int j;

    for (j = 0; j < NVAR; j++)
        v[j] = *vsgn[j] ? -*var[j] : *var[j];
}

/* Calcul du zéro-cycle défini par v. */
static inline void compute_zc (long v[NVAR], long p[4], long s[4][4],
                               long disc3, double complex zc[3][4])
{
    int j;
    double t;              /* Racine du covariant cubique.                  */
    double degen[6];       /* Membre dégénéré du pinceau de coniques.       */
    double evec[3][3];     /* Base propre de degen.                         */
    double r;              /* Quotient des valeurs propres de degen.        */
    double y;              /* Ordonnée dans la base propre de degen.        */
    double w[6];           /* Conique v[0..5] dans la base propre de degen. */
    double a, b1, b2, c1, c2;
    double complex x[2];   /* Abscisses dans la base propre de degen.       */

    t = root3 (p, s, disc3);
    for (j = 0; j < 6; j++)
        degen[j] = t * v[j] + v[j + 6];

    r = diagonalize (degen, evec);
    y = sqrt (fabs (r));

    w[0] = apply1 (v, evec, 0);
    w[1] = apply2 (v, evec, 0, 1);
    w[2] = apply2 (v, evec, 0, 2);
    w[3] = apply1 (v, evec, 1);
    w[4] = apply2 (v, evec, 1, 2);
    w[5] = apply1 (v, evec, 2);

    a = w[0];
    b1 = w[2] / 2;
    b2 = w[1] * y / 2;
    c1 = w[3] * r + w[5];
    c2 = w[4] * y;

    if (r >= 0) {
        roots2 (a, b1 + b2, c1 + c2, x);
        fill_zc (evec, x[0], y, zc, 0);
        fill_zc (evec, x[1], y, zc, 1);

        roots2 (a, b1 - b2, c1 - c2, x);
        fill_zc (evec, x[0], -y, zc, 2);
        fill_zc (evec, x[1], -y, zc, 3);
    }
    else {
        croots2 (a, b1 + b2 * I, c1 + c2 * I, x);
        fill_zc (evec, x[0], y * I, zc, 0);
        fill_zc (evec, x[1], y * I, zc, 1);

        croots2 (a, b1 - b2 * I, c1 - c2 * I, x);
        fill_zc (evec, x[0], -y * I, zc, 2);
        fill_zc (evec, x[1], -y * I, zc, 3);
    }
}

/* Calcul du covariant 3x3. */
static inline void compute_cov3 (double complex zc[3][4], double cov3[6])
{
    int i, i1, i2, j, j1, j2, k;
    double complex m2[3][6];   /* Mineurs 2x2.                        */
    double complex m3[4];      /* Mineurs 3x3.                        */
    double m3abs2[4];          /* Carrés des modules des mineurs 3x3. */
    double s[6];               /* Produits m3abs2[i] * m3abs2[j].     */

    for (i = 0; i < 3; i++) {
        i1 = (i + 1) % 3;
        i2 = (i + 2) % 3;
        j = 0;
        for (j1 = 0; j1 < 3; j1++)
            for (j2 = j1 + 1; j2 < 4; j2++) {
                m2[i][j] = zc[i1][j1] * zc[i2][j2] - zc[i2][j1] * zc[i1][j2];
                j++;
            }
    }
    for (j = 0; j < 4; j++)
        m3[j] = 0;
    for (i = 0; i < 3; i++) {
        m3[0] += zc[i][1] * m2[i][5];
        m3[1] += zc[i][0] * m2[i][5];
        m3[2] += zc[i][0] * m2[i][4];
        m3[3] += zc[i][0] * m2[i][3];
    }
    for (j = 0; j < 4; j++)
        m3abs2[j] = csprod (m3[j], m3[j]);

    j = 0;
    for (j1 = 0; j1 < 3; j1++)
        for (j2 = j1 + 1; j2 < 4; j2++) {
            s[j] = m3abs2[j1] * m3abs2[j2];
            j++;
        }
    k = 0;
    for (i1 = 0; i1 < 3; i1++)
        for (i2 = i1; i2 < 3; i2++) {
            cov3[k] = 0;
            for (j = 0; j < 6; j++)
                cov3[k] += s[j] * csprod (m2[i1][j], m2[i2][j]);
            k++;
        }
}

/* Conditions de réduction 3x3. On remplace l'inégalité '>' par un
 * test relâché qui tient compte des erreurs d'arrondi. */
static inline int red3 (double cov3[6])
{
    double n11, n12, n22;

    if (relaxed_gt (cov3[0], cov3[3]) || relaxed_gt (cov3[0], cov3[5]) ||
        relaxed_gt (2 * fabs (cov3[1]), cov3[0]) ||
        relaxed_gt (2 * fabs (cov3[2]), cov3[0]))
        return 0;

    n11 = cov3[0] * cov3[3] - cov3[1] * cov3[1];
    n22 = cov3[0] * cov3[5] - cov3[2] * cov3[2];

    if (relaxed_gt (n11, n22))
        return 0;

    n12 = cov3[0] * cov3[4] - cov3[2] * cov3[1];

    if (relaxed_gt (2 * fabs (n12), n11))
        return 0;

    if (n12 > 0) {
        if (relaxed_gt (cov3[0], cov3[3] + cov3[5] - 2 * cov3[4]))
            return 0;
    }
    else
        if (relaxed_gt (cov3[0], cov3[3] + cov3[5] + 2 * cov3[4] +
                        mindd (0, cov3[0] - 2 * (cov3[1] + cov3[2]))))
            return 0;

    return 1;
}

/* Calcule le résultant et vérifie l'irréductibilité. */
static inline int irred (long v[NVAR], long p[4], long r[5])
{
    pari_sp av = avma;
    long d[6][6];   /* Déterminants a_ij * b_kl - a_kl * b_ij. */

    if (lg (nfrootsQ (make_pol (p, 3))) > 1)
        return gc_bool (av, 0);

    d[0][1] = v[0] * v[7] - v[1] * v[6];
    d[0][3] = v[0] * v[9] - v[3] * v[6];
    d[1][3] = v[1] * v[9] - v[3] * v[7];

    if (!(r[0] = d[0][3] * d[0][3] - d[0][1] * d[1][3]))
        return gc_bool (av, 0);

    d[3][4] = v[3] * v[10] - v[4] * v[9];
    d[3][5] = v[3] * v[11] - v[5] * v[9];
    d[4][5] = v[4] * v[11] - v[5] * v[10];

    if (!(r[4] = d[3][5] * d[3][5] - d[3][4] * d[4][5]))
        return gc_bool (av, 0);

    d[0][4] = v[0] * v[10] - v[4] * v[6];
    d[1][2] = v[1] * v[8]  - v[2] * v[7];
    d[1][5] = v[1] * v[11] - v[5] * v[7];
    d[2][3] = v[2] * v[9]  - v[3] * v[8];
    d[2][4] = v[2] * v[10] - v[4] * v[8];

    r[1] = d[1][2] * d[1][3] + 2 * d[0][3] * d[2][3]
        + d[0][1] * d[3][4] - d[0][4] * d[1][3];
    r[3] = d[2][4] * d[3][4] - 2 * d[2][3] * d[3][5]
        + d[1][3] * d[4][5] - d[1][5] * d[3][4];
    r[2] = d[2][3] * d[2][3] + d[1][3] * d[1][5]
        + d[0][4] * d[3][4] - 2 * d[0][3] * d[3][5]
        - d[1][2] * d[3][4] - d[1][3] * d[2][4];

    if (r[0] + r[2] + r[4] == r[1] + r[3] ||
        r[0] + r[2] + r[4] == -(r[1] + r[3]))
        return gc_bool (av, 0);

    if (!(d[0][3] * d[3][4] * d[3][4] - d[3][5] * d[1][3] * d[1][3]
          + d[1][3] * d[2][3] * d[3][4]))
        return gc_bool (av, 0);

    if (lg (nfrootsQ (make_pol (r, 4))) > 1)
        return gc_bool (av, 0);

    return gc_bool (av, 1);
}

/* Vérifie la maximalité. */
static inline int maximal (long disc3, long r[5])
{
    pari_sp av = avma;

    if (-3 * itos (nfdisc (make_pol (r, 4))) != disc3)
        return gc_bool (av, 0);

    return gc_bool (av, 1);
}

/* Calcule une racine réelle du covariant cubique par la méthode de
 * Cardan. */
static inline double root3 (long p[4], long s[4][4], long disc3)
{
    double a = 4 * (9 * p[0] * (s[1][2] - 3 * s[0][3]) - 2 * p[1] * s[1][1]);
    double b2 = 16 * 9 * (double) s[0][0] * disc3;
    double b = sqrt (fabs (b2));

    if (disc3 >= 0)
        return (cbrt (a + b) + cbrt (a - b) - 2 * p[1]) / (6 * p[0]);
    else
        return (cbrt (sqrt (a * a - b2)) * cos (atan2 (b, a) / 3)
                - p[1]) / (3 * p[0]);
}

/* Reçoit un tableau de longueur 6 correspondant à une matrice
 * symétrique dégénérée. Calcule les vecteurs propres et les valeurs
 * propres non nulles lambda1 et lambda2, et renvoie le quotient
 * -lambda2 / lambda1. */
static inline double diagonalize (double degen[6], double evec[3][3])
{
    double hcom4[6];   /* Termes de 4 * comat (degen - lambda * id)
                        * qui ne dépendent pas de lambda. */
    double dcom4[3];   /* Termes diagonaux de la même comatrice. */
    double lambda1, lambda2;
    double b, c, rdisc;

    set_hcom4 (degen, hcom4);
    set_dcom4 (degen, hcom4, dcom4);

    find_evec (degen, hcom4, dcom4, evec, 0);

    b = degen[0] + degen[3] + degen[5];
    c = dcom4[0] + dcom4[1] + dcom4[2];
    rdisc = sqrt (b * b - c);

    lambda1 = (b + rdisc) / 2;
    lambda2 = (b - rdisc) / 2;

    degen[0] -= lambda1;
    degen[3] -= lambda1;
    degen[5] -= lambda1;
    set_dcom4 (degen, hcom4, dcom4);

    find_evec (degen, hcom4, dcom4, evec, 1);

    degen[0] += rdisc;
    degen[3] += rdisc;
    degen[5] += rdisc;
    set_dcom4 (degen, hcom4, dcom4);

    find_evec (degen, hcom4, dcom4, evec, 2);

    return -lambda2 / lambda1;
}

/* Évaluation de la forme quadratique correspondant aux 6 premiers
 * éléments de v en le j-ième vecteur propre de degen. */
static inline double apply1 (long v[6], double evec[3][3], int j)
{
    double l1, l2, l3;

    l1 = v[0] * evec[0][j] + v[1] * evec[1][j] + v[2] * evec[2][j];
    l2 = v[3] * evec[1][j] + v[4] * evec[2][j];
    l3 = v[5] * evec[2][j];

    return evec[0][j] * l1 + evec[1][j] * l2 + evec[2][j] * l3;
}

/* Évaluation de la forme bilinéaire correspondant aux 6 premiers
 * éléments de v en les j1-ième et j2-ième vecteurs propres de
 * degen. */
static inline double apply2 (long v[6], double evec[3][3],  int j1, int j2)
{
    double l1, l2, l3;

    l1 = 2 * v[0] * evec[0][j2] + v[1] * evec[1][j2] + v[2] * evec[2][j2];
    l2 = 2 * v[3] * evec[1][j2] + v[1] * evec[0][j2] + v[4] * evec[2][j2];
    l3 = 2 * v[5] * evec[2][j2] + v[2] * evec[0][j2] + v[4] * evec[1][j2];

    return evec[0][j1] * l1 + evec[1][j1] * l2 + evec[2][j1] * l3;
}

/* Étant donnés trois nombres réels a, b, c, calcule les racines
 * complexes du polynôme a * x^2 + 2 * b * x + c. */
static inline void roots2 (double a, double b, double c, double complex z[2])
{
    double disc = b * b - a * c;
    double x = -b / a;
    double y = sqrt (fabs (disc)) / a;

    if (disc >= 0) {
        z[0] = x + y;
        z[1] = x - y;
    }
    else {
        z[0] = x + y * I;
        z[1] = x - y * I;
    }
}

/* Étant donnés trois nombres complexes a, b, c, calcule les racines
 * complexes du polynôme a * x^2 + 2 * b * x + c. */
static inline void croots2 (double complex a, double complex b,
                            double complex c, double complex z[2])
{
    double complex disc = b * b - a * c;
    double complex x = -b / a;
    double complex y = csqrt (disc) / a;

    z[0] = x + y;
    z[1] = x - y;
}

/* Reçoit les coordonnées affines du j-ième point du zéro-cycle dans
 * la base propre de degen. Calcule les coordonnées projectives du
 * point dans la base d'origine et les enregistre dans zc. */
static inline void fill_zc (double evec[3][3], double complex x,
                            double complex y, double complex zc[3][4], int j)
{
    int i;

    for (i = 0; i < 3; i++)
        zc[i][j] = evec[i][0] * x + evec[i][1] * y + evec[i][2];
}

/* Mise à jour de hcom4. */
static inline void set_hcom4 (double degen[6], double hcom4[6])
{
    hcom4[0] = -degen[4] * degen[4];
    hcom4[3] = -degen[2] * degen[2];
    hcom4[5] = -degen[1] * degen[1];
    hcom4[1] = degen[2] * degen[4];
    hcom4[2] = degen[1] * degen[4];
    hcom4[4] = degen[1] * degen[2];
}

/* Mise à jour de dcom4. */
static inline void set_dcom4 (double degen[6],
                              double hcom4[6], double dcom4[3])
{
    dcom4[0] = hcom4[0] + 4 * degen[3] * degen[5];
    dcom4[1] = hcom4[3] + 4 * degen[0] * degen[5];
    dcom4[2] = hcom4[5] + 4 * degen[0] * degen[3];
}

/* Calcul du j-ième vecteur propre de degen. */
static void find_evec (double degen[6], double hcom4[6], double dcom4[3],
                       double evec[3][3], int j)
{
    int i, kmax;
    double norm;

    kmax = (fabs (dcom4[0]) < fabs (dcom4[1])) ?
        ((fabs (dcom4[1]) < fabs (dcom4[2])) ? 2 : 1) :
        ((fabs (dcom4[0]) < fabs (dcom4[2])) ? 2 : 0);

    switch (kmax) {
    case 0:
        evec[0][j] = dcom4[0];
        evec[1][j] = hcom4[1] - 2 * degen[1] * degen[5];
        evec[2][j] = hcom4[2] - 2 * degen[2] * degen[3];
        break;
    case 1:
        evec[0][j] = hcom4[1] - 2 * degen[1] * degen[5];
        evec[1][j] = dcom4[1];
        evec[2][j] = hcom4[4] - 2 * degen[0] * degen[4];
        break;
    case 2:
        evec[0][j] = hcom4[2] - 2 * degen[2] * degen[3];
        evec[1][j] = hcom4[4] - 2 * degen[0] * degen[4];
        evec[2][j] = dcom4[2];
        break;
    }
    norm = 0;
    for (i = 0; i < 3; i++)
        norm += evec[i][j] * evec[i][j];
    norm = sqrt (norm);

    for (i = 0; i < 3; i++)
        evec[i][j] /= norm;
}

/* Compte les éléments présents dans le fichier temporaire tmpfp. Si
 * le pointeur addresses n'est pas nul, enregistre les adresses des
 * éléments dans l'espace mémoire correspondant. */
static long count_elements_tmpfp (FILE *tmpfp, struct addr *addresses)
{
    long ecnt = 0;
    struct elt e;

    while (fread (&e, sizeof e, 1, tmpfp) == 1) {
        if (addresses) {
            addresses[ecnt].disc3 = e.disc3;
            addresses[ecnt].tmpfp = tmpfp;
            addresses[ecnt].offset = ecnt * sizeof e;
        }
        ecnt++;
    }
    return ecnt;
}

/* Compte les éléments présents dans les fichiers temporaires. Si le
 * pointeur addresses n'est pas nul, enregistre les adresses des
 * éléments dans l'espace mémoire correspondant. */
static long count_elements (struct addr *addresses)
{
    DIR *cwd;
    struct dirent *entry;
    long ecnt = 0;

    if (!(cwd = opendir (".")))
        exit (1);

    while ((entry = readdir (cwd))) {
        if (strncmp (entry->d_name, "tmp_", 4))
            continue;

        if (!(tmpfp = fopen (entry->d_name, "rb")))
            exit (1);

        ecnt += count_elements_tmpfp (tmpfp,
                                      addresses ? addresses + ecnt : NULL);
        if (!addresses)
            fclose (tmpfp);
    }
    closedir (cwd);
    return ecnt;
}

/* Lecture de l'élément à l'adresse a. */
static void read_element (struct addr *a, struct elt *e)
{
    fseek (a->tmpfp, a->offset, SEEK_SET);
    if (fread (e, sizeof *e, 1, a->tmpfp) < 1)
        exit (1);
}

/* Tri des adresses par discriminant croissant. */
static int compare_addresses (const void *p1, const void *p2)
{
    const struct addr *a1 = p1;
    const struct addr *a2 = p2;

    if (a1->disc3 > a2->disc3)
        return -1;
    else if (a1->disc3 < a2->disc3)
        return 1;
    else
        return 0;
}

/* Reçoit dcnt éléments de même discriminant, détecte ceux qui
 * définissent des corps de nombres isomorphes, et les indique dans le
 * tableau d. */
static void mark_duplicates (struct elt *e, int *d, int dcnt)
{
    pari_sp av;
    int i, j;
    static GEN *g;   /* Polynômes ou structures nf associées à e. */

    if (dcnt == 1) {
        d[0] = 0;
        return;
    }
    av = avma;
    if (!(g = realloc (g, dcnt * sizeof g[0])))
        exit (1);

    for (i = 0; i < dcnt; i++) {
        d[i] = 0;
        g[i] = make_pol (e[i].r, 4);

        for (j = 0; j < i; j++) {
            if (d[j])
                continue;

            if (typ (g[j]) == t_POL)
                g[j] = nfinitall (g[j], nf_RED | nf_ORIG, DEFAULTPREC);

            if (typ (nfisisom (g[j], g[i])) == t_VEC)
                d[i] = 1;
        }
    }
    set_avma (av);
}

/* Écriture d'un élément dans le fichier outfile. */
static void write_elt (FILE *outfile, struct elt *e)
{
    int j;

    fputs ("v = ", outfile);
    for (j = 0; j < NVAR; j++)
        fprintf (outfile, "%ld, ", e->v[j]);

    fputs ("\np = ", outfile);
    for (j = 0; j < 4; j++)
        fprintf (outfile, "%ld, ", e->p[j]);

    fputs ("\nr = ", outfile);
    for (j = 0; j < 5; j++)
        fprintf (outfile, "%ld, ", e->r[j]);

    fputs ("\n\n", outfile);
}

/* Lecture des adresses dont l'opposé du triple du discriminant est
 * disc3 et écriture des éléments correspondants dans le fichier
 * outfile. */
static int write_disc3 (struct addr *addresses, long ecnt, long disc3,
                        FILE *outfile)
{
    int i, dcnt;
    static struct elt *estore;   /* Stockage des éléments.       */
    static int *dstore;          /* Identification des doublons. */
    static int store_length;

    for (dcnt = 0; dcnt < ecnt && addresses[dcnt].disc3 == disc3; dcnt++) {
        if (dcnt == store_length) {
            store_length += 32;
            estore = realloc (estore, store_length * sizeof estore[0]);
            dstore = realloc (dstore, store_length * sizeof dstore[0]);
            if (!estore || !dstore)
                exit (1);
        }
        read_element (&addresses[dcnt], &estore[dcnt]);
    }
    mark_duplicates (estore, dstore, dcnt);

    fprintf (outfile, "disc = %ld :\n", -disc3 / 3);
    for (i = 0; i < dcnt; i++)
        if (!dstore[i])
            write_elt (outfile, &estore[i]);

    return dcnt;
}

/* Écriture des résultats dans le fichier "raw". */
static void write_res (void)
{
    long ecnt;
    int dcnt;
    struct addr *addresses;
    FILE *outfile;

    ecnt = count_elements (NULL);
    if (!(addresses = malloc (ecnt * sizeof addresses[0])))
        exit (1);
    count_elements (addresses);
    qsort (addresses, ecnt, sizeof addresses[0], compare_addresses);

    if (!(outfile = fopen ("raw", "w")))
        exit (1);

    while (ecnt) {
        dcnt = write_disc3 (addresses, ecnt, addresses->disc3, outfile);
        addresses += dcnt;
        ecnt -= dcnt;
    }
    fclose (outfile);
}

/* Somme les entiers dont les valeurs absolues sont contenues dans abs
 * et les signes dans sgn. */
static inline long sgnsum (long *abs, int *sgn, int length)
{
    int i;
    long res = 0;

    for (i = 0; i < length; i++)
        if (sgn[i])
            res -= abs[i];
        else
            res += abs[i];

    return res;
}

/* Produit scalaire de deux nombres complexes. */
static inline double csprod (double complex z1, double complex z2)
{
    return creal (z1) * creal (z2) + cimag (z1) * cimag (z2);
}

/* Reçoit deux réels positifs x1 et x2 et indique si x1 > x2 lorsqu'on
 * les tronque à WIDTH bits. */
static inline int relaxed_gt (double x1, double x2)
{
    double y1, y2;
    int n1, n2;

    if (!x1)
        return 0;
    else if (!x2)
        return 1;

    y1 = frexp (x1, &n1);
    y2 = frexp (x2, &n2);

    if (n1 > n2)
        return 1;
    else if (n1 < n2)
        return 0;

    if (y1 - y2 >= ldexp (1, -WIDTH))
        return 1;
    else
        return 0;
}

/* Construit un polynôme de degré n à partir de ses coefficients. */
static GEN make_pol (long *coefs, int n)
{
    int i;
    GEN pol = cgetg (n + 3, t_POL);

    pol[1] = evalvarn (0) | evalsigne (1);
    for (i = 0; i <= n; i++)
        gel (pol, n + 2 - i) = stoi (coefs[i]);

    return pol;
}
